@@ -1,8 +1,14 @@
 # solar.py
+import math
+
 import numpy as np
 from numpy.linalg import norm
-import math
-from passpredict.constants import DEG2RAD, RAD2DEG, AU_KM, R_EARTH
+from astropy.time import Time
+from astropy.coordinates import CartesianRepresentation, ITRS, get_sun
+
+from .constants import DEG2RAD, RAD2DEG, AU_KM, R_EARTH
+from .models import Sun
+
 
 def sun_pos(t):
     """Compute the Sun position vector
@@ -45,6 +51,10 @@ def sun_sat_angle(rsat, rsun):
     """
     crossproduct = np.cross(rsun, rsat, axisa=0, axisb=0).T
     sinzeta = norm(crossproduct, axis=0) / (norm(rsun, axis=0) * norm(rsat, axis=0))
+    if len(sinzeta.shape) > 0:
+        sinzeta[np.logical_and(1.0000003 > sinzeta, sinzeta > 1)] = 1.0  # fix some out of bounds warnings where sinzeta > 1.0
+    else: 
+        sinzeta = 1 if (1.0000003 > sinzeta > 1) else sinzeta
     return np.arcsin(sinzeta)
 
 
@@ -62,4 +72,23 @@ def sun_sat_orthogonal_distance(rsat, zeta):
 def is_sat_illuminated(rsat, rsun):
     zeta = sun_sat_angle(rsat, rsun)
     dist = sun_sat_orthogonal_distance(rsat, zeta)
-    return dist > R_EARTH
\ No newline at end of file
+    return dist > R_EARTH
+
+
+def compute_sun_data(t: Time) -> Sun:
+    """
+    Compute sun position data
+
+    Compute for each minute, then interpolate for each second
+    """
+    t_tmp = t[::60]
+    sun_tmp = get_sun(t_tmp)  # get astropy coordinates for sun in GCRS
+    sun_tmp = sun_tmp.transform_to(ITRS(obstime=t_tmp))  # transform to ECEF frame
+    sun_tmp = sun_tmp.data.xyz.to('km').value
+    sun_data = np.empty((3, t.size), dtype=np.float32)
+    for i in range(3):
+        sun_data[i] = np.interp(t.jd, t_tmp.jd, sun_tmp[i])
+    sun = Sun()
+    sun.time = t
+    sun.rECEF = sun_data.astype(np.float32)
+    return sun
