@@ -1,14 +1,25 @@
 import datetime
-import numpy as np
-from passpredict.constants import DAY_S, J2000
 import itertools
-import pytz
 
-tz_utc = pytz.utc
+import numpy as np
+from astropy.time import Time
+
+from .constants import DAY_S, J2000
+
+tz_utc = datetime.timezone.utc
+
+
+def jd2jc(jd1: float, jd2: float = 0.0) -> float:
+    """
+    Convert julian date to julian century
+    """
+    return ((jd1 - J2000) + jd2) / 36525.0
+
 
 def jd2utc1(jd, deltaUT1=0.0):
     pass
 
+
 def utc2tt(UTC, deltaAT=37.0, deltaUT1=0.0):
     """Compute terrestial time from UTC
 
@@ -28,7 +39,7 @@ def utc2tt(UTC, deltaAT=37.0, deltaUT1=0.0):
     # terrestial time in Julian Days
     JDtt = TAI + 32.184 / DAY_S
     # Convert JD to Julian Centuries
-    TT = (JDtt - J2000) / 36525
+    TT = jd2jc(JDtt)
     return TT
 
 
@@ -86,7 +97,7 @@ def julian_date2(yr, mo=1, dy=1, hr=0, mn=0, sec=0.0):
     return julian_day(yr, mo, dy) - 0.5 + (sec + mn * 60.0 + hr * 3600.0) / DAY_S
 
 
-def days2mdhms(year, days):
+def days2mdhms(year: int, days: float):
     """This procedure converts the day of the year, days, to the equivalent
     month, day, hour, minute and second.
     Args:
@@ -125,7 +136,68 @@ def days2mdhms(year, days):
     return mon, day, hr, minute, sec
 
 
-def invjday(jd):
+def invjday(jd: float):
+    """This procedure finds the year, month, day, hour, minute and second given the
+    julian date. jd can be ut1, tdt, tdb, etc.
+    Args:
+        jd : float, julian date, days from 4713 BCE
+    Outputs:
+        year : int, year between 1900 - 2100
+        mon : int, month between 1 - 12
+        day : int, day between 1 - 31
+        hr : int, hour between 0 - 23
+        min : int, minute between 0 - 59
+        sec : float, second between 0.0 - 59.999
+    References:
+        Vallado, 2007, 208, alg 22, ex 3-13
+        Rhodes, python-sgp4/sgp4/ext.py
+    """
+    # find year and days of the year
+    temp = jd - 2415019.5
+    tu = temp / 365.25  # julian centuries from 0 h jan 0, 1900
+    year = 1900 + int(tu // 1.0)
+    leapyrs = int(((year - 1901) * 0.25) // 1.0)
+    
+    # optional nudge by 8.64x10-7 sec to get even outputs
+    days  = temp - ((year - 1900) * 365.0 + leapyrs) + 0.00000000001
+    # check for case of beginning of a year
+    if (days < 1.0):
+        year    = year - 1
+        leapyrs = int(((year - 1901) * 0.25) // 1.0)
+        days    = temp - ((year - 1900) * 365.0 + leapyrs)
+    # find remaing data
+    mon, day, hr, minute, sec = days2mdhms(year, days)
+    sec -= 0.00000086400
+    return year, mon, day, hr, minute, sec
+
+
+def datetimes_to_datetimeary(year, mon, day, hr, minute, sec):
+    """Take arrays of datetime values and return array of datetime objects
+
+    Args:
+
+    Returns:
+        dtary : np.datetime64 (n)
+    """
+    n = len(year)
+    s = np.floor(sec).astype(int)
+    us = (np.mod(sec, 1) * 1e6).astype(int)
+    dtary = np.empty(n, dtype=object)
+    for i in range(len(year)):
+        # dtstr = f'{year[i]:4d}-{int(mon[i]):02d}-{int(day[i]):02d}T{int(hr[i]):02d}:{int(minute[i]):02d}:{sec[i]:05.2f}'
+        dtary[i] = datetime.datetime(year[i], int(mon[i]), int(day[i]), int(hr[i]), int(minute[i]), s[i], us[i], tzinfo=tz_utc)
+    return dtary
+
+
+def jday2datetime(jdt: float) -> datetime.datetime:
+    """Turn julian day into datetime object"""
+    datetuple = invjday(jdt)
+    yr, mo, date, hr, mn, sec = datetuple
+    sec, us = np.divmod(sec, 1)
+    return datetime.datetime(yr, mo, date, hr, mn, int(sec), int(us*1e6), tzinfo=tz_utc)
+    
+
+def invjday_array(jd):
     """This procedure finds the year, month, day, hour, minute and second given the
     julian date. jd can be ut1, tdt, tdb, etc.
     Args:
@@ -168,27 +240,9 @@ def invjday(jd):
     return year, mon, day, hr, minute, sec
 
 
-def datetimes_to_datetimeary(year, mon, day, hr, minute, sec):
-    """Take arrays of datetime values and return array of datetime objects
-
-    Args:
-
-    Returns:
-        dtary : np.datetime64 (n)
-    """
-    n = len(year)
-    s = np.floor(sec).astype(int)
-    us = (np.mod(sec, 1) * 1e6).astype(int)
-    dtary = np.empty(n, dtype=object)
-    for i in range(len(year)):
-        # dtstr = f'{year[i]:4d}-{int(mon[i]):02d}-{int(day[i]):02d}T{int(hr[i]):02d}:{int(minute[i]):02d}:{sec[i]:05.2f}'
-        dtary[i] = datetime.datetime(year[i], int(mon[i]), int(day[i]), int(hr[i]), int(minute[i]), s[i], us[i], tzinfo=tz_utc)
-    return dtary
-
-
-def jday2datetime(jdt):
+def jday2datetime_array(jdt_array):
     """Turn julian day into datetime object"""
-    datetuple = invjday(jdt)
+    datetuple = invjday_array(jdt)
     yr, mo, date, hr, mn, sec = datetuple
     sec, us = np.divmod(sec, 1)
     sec = sec.astype(int)
@@ -196,8 +250,6 @@ def jday2datetime(jdt):
     dt_array = np.empty(yr.size, dtype=object)
     for i, data in enumerate(zip(yr, mo, date, hr, mn, sec, us.astype(int))):
         dt_array[i] = datetime.datetime(*data, tzinfo=tz_utc)
-    if not isinstance(jdt, np.ndarray):
-        dt_array = dt_array[0]
     return dt_array
 
 
@@ -225,12 +277,23 @@ def jdt_tsince(tstart, tsince):
     return tstart + (tsince * 60.0) / 86400.0
 
 
-def truncate_datetime(dt):
+def truncate_datetime(dt, nearest='s', utc=True):
     """ Truncate datetime object to the previous second """
+    
     us = dt.microsecond
-    dt2 = dt - datetime.timedelta(microseconds=us)
-    return dt2
+    dt -= datetime.timedelta(microseconds=us)
+    if nearest in ('d', 'day'):
+        if utc is True:
+            dt = datetime.datetime(dt.year, dt.month, dt.day, tzinfo=tz_utc)
+        else:
+            dt = datetime.datetime(dt.year, dt.month, dt.day)
+    return dt
 
+def get_date(dt):
+    """ 
+    Return datetime object to 00:00:00 of that day
+    """
+    return datetime.datetime(dt.year, dt.month, dt.day, tzinfo=tz_utc)
 
 def datetime_linspace(datetime_start, datetime_end, dt_seconds):
     """
@@ -244,17 +307,29 @@ def datetime_linspace(datetime_start, datetime_end, dt_seconds):
         dtype="datetime64[s]"
     )
 
-if __name__ == "__main__":
-    julian_date = 2450383.09722222  # 1996-10-26 14:20:0
-    jd_array = np.arange(julian_date, julian_date + 3, 1/24.)
-    dt_array = jday2datetime(jd_array)
-    print(dt_array)
 
-    dt_array = jday2datetime(julian_date)
-    print(dt_array)
+def compute_time_array(dt_start: datetime.datetime, dt_end: datetime.datetime, dt_seconds: float) -> Time:
+    """
+    Create astropy Time object
+    """
+    jdt0 = julian_date(dt_start)
+    jdtf = julian_date(dt_end)
+    dt_days = dt_seconds/(24*60*60.0)
+    jd_array = np.arange(jdt0, jdtf, dt_days, dtype=float)
+    return Time(jd_array, format='jd')
+
+
+def compute_time_array_from_date(*a, **kw):
+    return time_array_from_date(*a, **kw)
+
+
+def time_array_from_date(date_start: datetime.date, date_end: datetime.date, dt_seconds: float) -> Time:
+    """
+    Create astropy Time object from python date
+    """
+    jdt0 = julian_day(date_start.year, date_start.month, date_start.day)
+    jdtf = julian_day(date_end.year, date_end.month, date_end.day)
+    dt_days = dt_seconds/(24*60*60.0)
+    jd_array = np.arange(jdt0, jdtf, dt_days, dtype=float)
+    return Time(jd_array, format='jd')
 
-    # now = datetime.datetime.now()
-    # then = now + datetime.timedelta(days=7)
-    # dt_seconds = datetime.timedelta(seconds=5)
-    # for d in range(now, then, dt_seconds):
-    #     print(d)
\ No newline at end of file
